/*
m_jbsoc_delete - Helper opcode to destroy JBSO objects

DESCRIPTION
The final operations to be performed when destroying a JBSO object.

SYNTAX
iSuccess m_jbsoc_delete SclassName, Sname

INITIALIZATION
iSuccess - 1 upon success, != otherwise
SclassName - Name of the class the object belongs to
Sname - Name of the object to delete

PERFORMANCE

CREDITS
Author: Jeanette C.
*/

opcode m_jbsoc_delete, i, SS
	SclassName, Sname xin

	; Initialise the return value
	iSuccess init $JBSO_STAT_OK ; reset when errors are encountered

	; Check the object name and existence
	if (strcmp(Sname, "Global") == 0) then
		prints("\nERROR: JBSO does not allow objects named Global")
		iSuccess = $JBSO_STAT_ERR
	else ; Sname is not Global
		SclassGlobal = strcat(strcat("JBSO.", SclassName), ".Global")
		if (chnget:i(strcat(SclassGlobal, ".Valid")) != $JBSO_STAT_OK) then
			prints("\nERROR: There is no JBSO type named %s", SclassName)
			iSuccess = $JBSO_STAT_ERR
		else ; class exists
			Sobject = strcat(strcat(strcat("JBSO.", SclassName), "."), Sname)
			if (chnget:i(strcat(Sobject, ".Valid")) != $JBSO_STAT_OK) then
				prints("\nERROR: There is no object called %s of type %s", Sname, SclassName)
				iSuccess = $JBSO_STAT_ERR
			endif ; Sname does not exist
		endif ; Class does not exist
	endif ; Name is global

	; Destroy the object, if it exists
	if (iSuccess == $JBSO_STAT_OK) then
		chnset(0, strcat(Sobject, ".Valid")) ; object not valid
		chnset(0, strcat(Sobject, ".Status")) ; unset Error status
		chnset("", strcat(Sobject, ".StatusString"))

		; Unset/unlink the ID
		iId = chnget:i(strcat(Sobject, ".ID"))
		chnset(0, strcat(Sobject, ".ID"))
		Sid = sprintf("%d", iId)
		chnset("", strcat(strcat(SclassGlobal, "."), Sid))
		iIdsTable = chnget:i(strcat(SclassGlobal, ".Ids"))
		tabw_i(-1, iId, iIdsTable)

		; Decrease JBSO and class object counters
		iClassCount = chnget:i(strcat(SclassGlobal, ".Count"))
		chnset((iClassCount -1), strcat(SclassGlobal, ".Count"))
		iGlobalCount = chnget:i("JBSO.Global.Count")
		chnset((iGlobalCount -1), "JBSO.Global.Count")
	endif

	; Return the success status
	xout(iSuccess)
endop
