/*
m_jbsoc_get_size - Return storage size for objects of a class/type

DESCRIPTION
Return the storage size for objects of the given class/type or $JBSO_STAT_ERR
on error.

SYNTAX
kSize m_jbsoc_get_size SclassName

INITIALIZATION

PERFORMANCE
kSize - Storage size of the class/type or $JBSO_STAT_ERR on error. 0 is a valid storage
	size, for objects that can't be saved to disk or have no data to save yet.
SclassName - Name of the class to check for storage size

CREDITS
Author: Jeanette C.
*/

opcode m_jbsoc_get_size, k, S
	SclassName xin

	; Initialise the local return value
	kSize init $JBSO_STAT_ERR

	; Create the type's global channel name
	SclassGlobal = strcat(strcat("JBSO.", SclassName), ".Global")

	if (chnget:k(strcat(SclassGlobal, ".Valid")) == $JBSO_STAT_OK) then ; the type exists
		kSize = chnget:k(strcat(SclassGlobal, ".Size"))
	endif

	; Return the size
	xout(kSize)
endop
