/*
m_jbsoc_new - Helper opcode to create JBSO objects

DESCRIPTION
This is a utility opcode meant only to be used in higher level UDOs to create
JBSO objects. It is highly recommended to use it in tandem with m_jbsoc_delete .

SYNTAX
iSuccess m_jbsoc_new SclassName, Sname, iVersionMajor, iVersionMinor

INITIALIZATION
iSuccess - 1 on successful setup, !=1 otherwise
SclassName - Name of the object's class
Sname - Name of the object to set up
iVersionMajor, iVersionMinor - Major and minor version number of the type

PERFORMANCE

CREDITS
Author: Jeanette C.
*/

opcode m_jbsoc_new, i, SSii
	SclassName, Sname, iVersionMajor, iVersionMinor xin

	; Initialise return value
	iSuccess init $JBSO_STAT_OK
	iSearchIndex init 0 ; find a free object ID if necessary

	; Only continue if the object name isn't Global
	if (strcmp(Sname, "Global") == 0) then
		prints("\nERROR: JBSO won't create an object named Global.")
		iSuccess = $JBSO_STAT_ERR
	else
		; Call m_jbso_init for good measure
		m_jbso_init
		; Check if the class exists
		SclassGlobal = strcat(strcat("JBSO.", SclassName), ".Global")
	
		if (chnget:i(strcat(SclassGlobal, ".Valid")) == 0) then ; initialise class
			chn_k(strcat(SclassGlobal, ".Valid"), 3)
			chnset($JBSO_STAT_OK, strcat(SclassGlobal, ".Valid"))
			chn_k(strcat(SclassGlobal, ".VersionMajor"), 3)
			chnset(iVersionMajor, strcat(SclassGlobal, ".VersionMajor"))
			chn_k(strcat(SclassGlobal, ".VersionMinor"), 3)
			chnset(iVersionMinor, strcat(SclassGlobal, ".VersionMinor"))
			iIdTable = ftgen(0, 0, 512, -7, -1, 512, -1)
			chn_k(strcat(SclassGlobal, ".Ids"), 3)
			chnset(iIdTable, strcat(SclassGlobal, ".Ids"))
		endif ; Clas set up

		; Object setup
		iClassCount = chnget:i(strcat(SclassGlobal, ".Count"))
		iGlobalCount = chnget:i("JBSO.Global.Count")

		; Create object channelname
		Sobject = strcat(strcat(strcat("JBSO.", SclassName), "."), Sname)

		; Check if an object of the given name already exists
		if (chnget:i(strcat(Sobject, ".Valid")) == $JBSO_STAT_OK) then
			prints("\nERROR: An object named %s of type %s already exists.", Sname, SclassName)
			iSuccess = $JBSO_STAT_ERR
		else
			; Set Ojbect ID and name
			iIdsTable = chnget:i(strcat(SclassGlobal, ".Ids"))
			if (tab_i(iClassCount, iIdsTable) == -1) then ; direct hit
				tabw_i(iClassCount, iClassCount, iIdsTable)
				Sid = sprintf("%d", (iClassCount -1))
				chn_S(strcat(strcat(SclassGlobal, "."), Sid), 3)
				chnset(Sname, strcat(strcat(SclassGlobal, "."), Sid))
				chn_k(strcat(Sobject, ".ID"), 3)
				chnset(iClassCount, strcat(Sobject, ".ID"))
				chnset((iClassCount + 1), strcat(SclassGlobal, ".Count"))
				chnset((iGlobalCount + 1), "JBSO.Global.Count")
			else ; look for an empty slot
				iNewId init 0
				while ((iNewId != -1) && (iSearchIndex < 512)) do
					iNewId = tab_i(iSearchIndex, iIdsTable)
					iSearchIndex += 1
				od
				if (iSearchIndex >= 512) then ; too many objects
					prints("\nERROR: can't create object %s of type %s, there are too many objects.", Sname, SclassName)
					iSuccess = $JBSO_STAT_ERR
				else ; an object ID was found
					tabw_i((iSearchIndex -1), (iSearchIndex -1), iIdsTable)
					Sid = sprintf("%d", (iSearchIndex -1))
					chn_S(strcat(strcat(SclassGlobal, "."), Sid), 3)
					chnset(Sname, strcat(strcat(SclassGlobal, "."), Sid))
					chn_k(strcat(Sobject, ".ID"), 3)
					chnset((iSearchIndex -1), strcat(Sobject, ".ID"))
					chnset((iClassCount + 1), strcat(SclassGlobal, ".Count"))
					chnset((iGlobalCount + 1), "JBSO.Global.Count")
				endif
			endif
		endif

		; Set basic object properties
		if (iSuccess == $JBSO_STAT_OK) then
			chn_k(strcat(Sobject, ".Valid"), 3)
			chnset($JBSO_STAT_OK, strcat(Sobject, ".Valid"))
			chn_k(strcat(Sobject, ".Status"), 3)
			chnset($JBSO_STAT_OK, strcat(Sobject, ".Status"))
			chn_S(strcat(Sobject, ".StatusString"), 3)
			chnset("", strcat(Sobject, ".StatusString"))
		endif ; set addition properties
	endif // Name of object not Global

	; Return success result
	xout(iSuccess)
endop
